@@ -2,7 +2,8 @@ use std::borrow::Cow;
 use std::cmp;
 use std::fmt::Write as _;
 use std::fs::{self, File};
-use std::io::{self, IsTerminal as _, Read as _, Write as _};
+use std::io::{self, IsTerminal as _, Read as _};
+use std::iter;
 use std::iter::zip;
 use std::num::NonZeroU8;
 use std::path::{Path, PathBuf};
@@ -24,8 +25,9 @@ use hyfetch::models::Config;
 #[cfg(feature = "macchina")]
 use hyfetch::neofetch_util::macchina_path;
 use hyfetch::neofetch_util::{self, add_pkg_path, fastfetch_path, get_distro_ascii, get_distro_name, literal_input, ColorAlignment, NEOFETCH_COLORS_AC, NEOFETCH_COLOR_PATTERNS, TEST_ASCII};
-use hyfetch::presets::{AssignLightness, Preset};
-use hyfetch::pride_month;
+use hyfetch::color_profile::{AssignLightness, ColorProfile};
+use hyfetch::presets::{Preset};
+use hyfetch::{pride_month, printc};
 use hyfetch::types::{AnsiMode, Backend, TerminalTheme};
 use hyfetch::utils::{get_cache_path, input};
 use hyfetch::font_logo::get_font_logo;
@@ -64,15 +66,12 @@ fn main() -> Result<()> {
     });
 
     if options.test_print {
-        let asc = get_distro_ascii(distro, backend).context("failed to get distro ascii")?;
-        writeln!(io::stdout(), "{asc}", asc = asc.asc)
-            .context("failed to write ascii to stdout")?;
+        println!("{asc}", asc = get_distro_ascii(distro, backend)?.asc);
         return Ok(());
     }
 
     if options.print_font_logo {
-        let logo = get_font_logo(backend).context("failed to get font logo")?;
-        writeln!(io::stdout(), "{}", logo).context("failed to write logo to stdout")?;
+        println!("{}", get_font_logo(backend)?);
         return Ok(());
     }
 
@@ -110,14 +109,9 @@ fn main() -> Result<()> {
 
     if show_pride_month && !config.pride_month_disable {
         pride_month::start_animation(color_mode).context("failed to draw pride month animation")?;
-        writeln!(
-            io::stdout(),
-            "\nHappy pride month!\n(You can always view the animation again with `hyfetch \
-             --june`)\n"
-        )
-        .context("failed to write message to stdout")?;
+        println!("\nHappy pride month!\n(You can always view the animation again with `hyfetch --june`)\n");
 
-        if !june_path.is_file() {
+        if !june_path.is_file() && !options.june {
             File::create(&june_path)
                 .with_context(|| format!("failed to create file {june_path:?}"))?;
         }
@@ -129,9 +123,43 @@ fn main() -> Result<()> {
     let backend = options.backend.unwrap_or(config.backend);
     let args = options.args.as_ref().or(config.args.as_ref());
 
+    fn parse_preset_string(preset_string: &str) -> Result<ColorProfile> {
+        if preset_string.contains('#') {
+            let colors: Vec<&str> = preset_string.split(',').map(|s| s.trim()).collect();
+            for color in &colors {
+                if !color.starts_with('#') ||
+                    (color.len() != 4 && color.len() != 7) ||
+                    !color[1..].chars().all(|c| c.is_ascii_hexdigit()) {
+                    return Err(anyhow::anyhow!("invalid hex color: {}", color));
+                }
+            }
+            ColorProfile::from_hex_colors(colors)
+                .context("failed to create color profile from hex")
+        } else if preset_string == "random" {
+            let mut rng = fastrand::Rng::new();
+            let preset = *rng
+                .choice(<Preset as VariantArray>::VARIANTS)
+                .expect("preset iterator should not be empty");
+            Ok(preset.color_profile())
+        } else {
+            use std::str::FromStr;
+            let preset = Preset::from_str(preset_string)
+                .with_context(|| {
+                    format!(
+                        "PRESET should be comma-separated hex colors or one of {{{presets}}}",
+                        presets = <Preset as VariantNames>::VARIANTS
+                            .iter()
+                            .chain(iter::once(&"random"))
+                            .join(",")
+                    )
+                })?;
+            Ok(preset.color_profile())
+        }
+    }
+
     // Get preset
-    let preset = options.preset.unwrap_or(config.preset);
-    let color_profile = preset.color_profile();
+    let preset_string = options.preset.as_deref().unwrap_or(&config.preset);
+    let color_profile = parse_preset_string(preset_string)?;
     debug!(?color_profile, "color profile");
 
     // Lighten
@@ -149,7 +177,14 @@ fn main() -> Result<()> {
     };
     debug!(?color_profile, "lightened color profile");
 
-    let asc = if let Some(path) = options.ascii_file {
+    let asc = if let Some(path_str) = config.custom_ascii_path {
+        let path = PathBuf::from(path_str);
+        RawAsciiArt {
+            asc: fs::read_to_string(&path)
+                .with_context(|| format!("failed to read ascii from {path:?}"))?,
+            fg: Vec::new(),
+        }
+    } else if let Some(path) = options.ascii_file {
         RawAsciiArt {
             asc: fs::read_to_string(&path)
                 .with_context(|| format!("failed to read ascii from {path:?}"))?,
@@ -207,9 +242,9 @@ fn det_bg() -> Result<Option<Srgb<u8>>, terminal_colorsaurus::Error> {
     }
 
     background_color(QueryOptions::default())
-        .map(|terminal_colorsaurus::Color { r, g, b }| Some(Srgb::new(r, g, b).into_format()))
+        .map(|terminal_colorsaurus::Color { r, g, b , .. }| Some(Srgb::new(r, g, b).into_format()))
         .or_else(|err| {
-            if matches!(err, terminal_colorsaurus::Error::UnsupportedTerminal) {
+            if matches!(err, terminal_colorsaurus::Error::UnsupportedTerminal(_)) {
                 Ok(None)
             } else {
                 Err(err)
@@ -236,10 +271,6 @@ fn create_config(
         } else if color_level.has_256 {
             AnsiMode::Ansi256
         } else if color_level.has_basic {
-            // unimplemented!(
-            //     "{mode} color mode not supported",
-            //     mode = AnsiMode::Ansi16.as_ref()
-            // );
             AnsiMode::Ansi256
         } else {
             unreachable!();
@@ -279,13 +310,8 @@ fn create_config(
             .expect("`option_counter` should not overflow `u8`");
     }
 
-    fn print_title_prompt(
-        option_counter: NonZeroU8,
-        prompt: &str,
-        color_mode: AnsiMode,
-    ) -> Result<()> {
-        printc(format!("&a{option_counter}. {prompt}"), color_mode)
-            .context("failed to print prompt")
+    fn print_title_prompt(option_counter: NonZeroU8, prompt: &str) {
+        printc!("&a{option_counter}. {prompt}");
     }
 
     //////////////////////////////
@@ -295,14 +321,8 @@ fn create_config(
         let (Width(term_w), Height(term_h)) = terminal_size().context("failed to get terminal size")?;
         let (term_w_min, term_h_min) = ((asc.w as u32 * 2 + 4).clamp(0, u16::MAX.into()) as u16, 30);
         if term_w < term_w_min || term_h < term_h_min {
-            printc(
-                format!(
-                    "&cWarning: Your terminal is too small ({term_w} * {term_h}).\nPlease resize \
-                     it to at least ({term_w_min} * {term_h_min}) for better experience."
-                ),
-                color_mode,
-            )
-            .context("failed to print message")?;
+            printc!("&cWarning: Your terminal is too small ({term_w} * {term_h}).\n\
+                     Please resize it to at least ({term_w_min} * {term_h_min}) for better experience.");
             input(Some("Press enter to continue...")).context("failed to read input")?;
         }
     }
@@ -347,18 +367,14 @@ fn create_config(
             (t - a) * ((d - c) / (b - a)) + c
         }
 
-        {
-            let label = format!(
-                "{label:^term_w$}",
-                label = "8bit Color Testing",
-                term_w = usize::from(term_w)
-            );
+        let print_color_testing = |label: &str, mode: AnsiMode| {
+            let label = format!("{label:^term_w$}", term_w = usize::from(term_w));
             let line = zip(gradient.iter(), label.chars()).fold(
                 String::new(),
                 |mut s, (&rgb_f32_color, t)| {
                     let rgb_u8_color = Srgb::<u8>::from_linear(rgb_f32_color);
                     let back = rgb_u8_color
-                        .to_ansi_string(AnsiMode::Ansi256, ForegroundBackground::Background);
+                        .to_ansi_string(mode, ForegroundBackground::Background);
                     let fore = rgb_u8_color
                         .contrast_grayscale()
                         .to_ansi_string(AnsiMode::Ansi256, ForegroundBackground::Foreground);
@@ -366,42 +382,15 @@ fn create_config(
                     s
                 },
             );
-            printc(line, AnsiMode::Ansi256).context("failed to print 8-bit color test line")?;
-        }
-        {
-            let label = format!(
-                "{label:^term_w$}",
-                label = "RGB Color Testing",
-                term_w = usize::from(term_w)
-            );
-            let line = zip(gradient.iter(), label.chars()).fold(
-                String::new(),
-                |mut s, (&rgb_f32_color, t)| {
-                    let rgb_u8_color = Srgb::<u8>::from_linear(rgb_f32_color);
-                    let back = rgb_u8_color
-                        .to_ansi_string(AnsiMode::Rgb, ForegroundBackground::Background);
-                    let fore = rgb_u8_color
-                        .contrast_grayscale()
-                        .to_ansi_string(AnsiMode::Ansi256, ForegroundBackground::Foreground);
-                    write!(s, "{back}{fore}{t}").unwrap();
-                    s
-                },
-            );
-            printc(line, AnsiMode::Rgb).context("failed to print RGB color test line")?;
-        }
+            printc!("{line}");
+        };
 
-        writeln!(io::stdout()).context("failed to write to stdout")?;
-        print_title_prompt(
-            option_counter,
-            "Which &bcolor system &ado you want to use?",
-            color_mode,
-        )
-        .context("failed to print title prompt")?;
-        writeln!(
-            io::stdout(),
-            "(If you can't see colors under \"RGB Color Testing\", please choose 8bit)\n"
-        )
-        .context("failed to write message to stdout")?;
+        print_color_testing("8bit Color Testing", AnsiMode::Ansi256);
+        print_color_testing("RGB Color Testing", AnsiMode::Rgb);
+
+        println!();
+        print_title_prompt(option_counter, "Which &bcolor system &ado you want to use?");
+        println!("(If you can't see colors under \"RGB Color Testing\", please choose 8bit)\n");
 
         let choice = literal_input(
             "Your choice?",
@@ -434,20 +423,14 @@ fn create_config(
 
         clear_screen(Some(&title), color_mode, debug_mode).context("failed to clear screen")?;
 
-        print_title_prompt(
-            option_counter,
-            "Is your terminal in &blight mode&~ or &4dark mode&~?",
-            color_mode,
-        )
-        .context("failed to print title prompt")?;
+        print_title_prompt(option_counter, "Is your terminal in &blight mode&~ or &4dark mode&~?");
         let choice = literal_input(
             "",
             TerminalTheme::VARIANTS,
             TerminalTheme::Dark.as_ref(),
             true,
             color_mode,
-        )
-        .context("failed to ask for choice input")?;
+        )?;
         Ok((
             choice.parse().expect("selected theme should be valid"),
             "Selected background color",
@@ -518,18 +501,12 @@ fn create_config(
 
     let print_flag_page = |page, page_num: u8| -> Result<()> {
         clear_screen(Some(&title), color_mode, debug_mode).context("failed to clear screen")?;
-        print_title_prompt(option_counter, "Let's choose a flag!", color_mode)
-            .context("failed to print title prompt")?;
-        writeln!(
-            io::stdout(),
-            "Available flag presets:\nPage: {page_num} of {num_pages}\n",
-            page_num = page_num.checked_add(1).unwrap()
-        )
-        .context("failed to write header to stdout")?;
+        print_title_prompt(option_counter, "Let's choose a flag!");
+        println!("Available flag presets:\nPage: {page_num} of {num_pages}\n", page_num = page_num + 1);
         for &row in page {
             print_flag_row(row, color_mode).context("failed to print flag row")?;
         }
-        writeln!(io::stdout()).context("failed to write to stdout")?;
+        println!();
         Ok(())
     };
 
@@ -541,7 +518,7 @@ fn create_config(
             }
             printc(line.join("  "), color_mode).context("failed to print line")?;
         }
-        writeln!(io::stdout()).context("failed to write to stdout")?;
+        println!();
         Ok(())
     }
 
@@ -566,16 +543,9 @@ fn create_config(
         let mut opts: Vec<&str> = <Preset as VariantNames>::VARIANTS.into();
         opts.extend(["next", "n", "prev", "p"]);
 
-        writeln!(
-            io::stdout(),
-            "Enter '[n]ext' to go to the next page and '[p]rev' to go to the previous page."
-        )
-        .context("failed to write message to stdout")?;
+        println!("Enter '[n]ext' to go to the next page and '[p]rev' to go to the previous page.");
         let selection = literal_input(
-            format!(
-                "Which {preset} do you want to use? ",
-                preset = preset_default_colored
-            ),
+            format!("Which {preset_default_colored} do you want to use? "),
             &opts[..],
             Preset::Rainbow.as_ref(),
             false,
@@ -624,22 +594,15 @@ fn create_config(
 
     let select_lightness = || -> Result<Lightness> {
         clear_screen(Some(&title), color_mode, debug_mode).context("failed to clear screen")?;
-        print_title_prompt(
-            option_counter,
-            "Let's adjust the color brightness!",
-            color_mode,
-        )
-        .context("failed to print title prompt")?;
-        writeln!(
-            io::stdout(),
+        print_title_prompt(option_counter, "Let's adjust the color brightness!");
+        println!(
             "The colors might be a little bit too {bright_dark} for {light_dark} mode.\n",
             bright_dark = match theme {
                 TerminalTheme::Light => "bright",
                 TerminalTheme::Dark => "dark",
             },
             light_dark = theme.as_ref()
-        )
-        .context("failed to write message to stdout")?;
+        );
 
         let color_align = ColorAlignment::Horizontal;
 
@@ -697,14 +660,10 @@ fn create_config(
         }
 
         loop {
-            writeln!(
-                io::stdout(),
-                "\nWhich brightness level looks the best? (Default: {default:.0}% for \
-                 {light_dark} mode)",
+            println!("\nWhich brightness level looks the best? (Default: {default:.0}% for {light_dark} mode)",
                 default = f32::from(default_lightness) * 100.0,
                 light_dark = theme.as_ref()
-            )
-            .context("failed to write prompt to stdout")?;
+            );
             let lightness = input(Some("> "))
                 .context("failed to read input")?
                 .trim()
@@ -716,12 +675,7 @@ fn create_config(
                 },
                 Err(err) => {
                     debug!(%err, "could not parse lightness");
-                    printc(
-                        "&cUnable to parse lightness value, please enter a lightness value such \
-                         as 45%, .45, or 45",
-                        color_mode,
-                    )
-                    .context("failed to print message")?;
+                    printc!("&cUnable to parse lightness value, please enter a lightness value such as 45%, .45, or 45");
                 },
             }
         }
@@ -769,8 +723,11 @@ fn create_config(
     // get distro string and convert it into the enum, neofetch friendly format, so we can check for small logos with the {distro}_small neofetch naming scheme.
     let tmp_dst = get_distro_name(backend).or_else(|_| get_distro_name(Backend::Neofetch)).context("failed to get distro name")?;
     let detected_dst = Some(distro.map_or_else(
-        || format!("{:?}", Distro::detect(tmp_dst).unwrap()),
-        |d| d.to_string(),
+        || {
+            Distro::detect(&tmp_dst)
+                .map_or("".to_string(), |d| format!("{:?}", d).to_lowercase())
+        },
+        |d| d.to_string().to_lowercase(),
     ));
 
     // in case someone specified {distro}_small already in the --distro arg
@@ -845,16 +802,11 @@ fn create_config(
                     }
                     printc(line.join("                 "), color_mode).context("failed to print ascii line")?; 
                 }
-        
-                writeln!(io::stdout()).context("failed to write to stdout")?;
+
+                println!();
             }
 
-            print_title_prompt(
-                option_counter,
-                "Do you want the default logo, or the small logo?",
-                color_mode,
-            )
-            .context("failed to print title prompt")?;
+            print_title_prompt(option_counter, "Do you want the default logo, or the small logo?");
             let opts: Vec<Cow<str>> = ["default", "small"].map(Into::into).into();
             let choice = literal_input("Your choice?", &opts[..], "default", true, color_mode)
                 .context("failed to ask for choice input")
@@ -969,21 +921,11 @@ fn create_config(
                 }
                 printc(line.join("    "), color_mode).context("failed to print ascii line")?;
             }
-            writeln!(io::stdout()).context("failed to write to stdout")?;
+            println!();
         }
 
-        print_title_prompt(
-            option_counter,
-            "Let's choose a color arrangement!",
-            color_mode,
-        )
-        .context("failed to print title prompt")?;
-        writeln!(
-            io::stdout(),
-            "You can choose standard horizontal or vertical alignment, or use one of the random \
-             color schemes.\nYou can type \"roll\" to randomize again.\n"
-        )
-        .context("failed to write message to stdout")?;
+        print_title_prompt(option_counter, "Let's choose a color arrangement!");
+        println!("You can choose standard horizontal or vertical alignment, or use one of the random color schemes.\nYou can type \"roll\" to randomize again.\n");
         let mut opts: Vec<Cow<str>> = ["horizontal", "vertical", "roll"].map(Into::into).into();
         opts.extend((0..random_count).map(|i| format!("random{i}").into()));
         let choice = literal_input("Your choice?", &opts[..], "horizontal", true, color_mode)
@@ -1022,42 +964,27 @@ fn create_config(
 
     let select_backend = || -> Result<Backend> {
         clear_screen(Some(&title), color_mode, debug_mode).context("failed to clear screen")?;
-        print_title_prompt(option_counter, "Select a *fetch backend", color_mode)
-            .context("failed to print title prompt")?;
+        print_title_prompt(option_counter, "Select a *fetch backend");
 
         // Check if fastfetch is installed
         let fastfetch_path = fastfetch_path().ok();
 
         // Check if macchina is installed
         #[cfg(feature = "macchina")]
-        let macchina_path = macchina_path().context("failed to get macchina path")?;
+        let macchina_path = macchina_path().unwrap_or(None);
 
-        printc(
-            "- &bneofetch&r: Written in bash, &nbest compatibility&r on Unix systems",
-            color_mode,
-        )
-        .context("failed to print message")?;
-        printc(
-            format!(
-                "- &bfastfetch&r: Written in C, &nbest performance&r {installed_not_installed}",
-                installed_not_installed = fastfetch_path
-                    .map(|path| format!("&a(Installed at {path})", path = path.display()))
-                    .unwrap_or_else(|| "&c(Not installed)".to_owned())
-            ),
-            color_mode,
-        )
-        .context("failed to print message")?;
+        printc!("- &bneofetch&r: Written in bash, &nbest compatibility&r on Unix systems");
+        printc!("- &bfastfetch&r: Written in C, &nbest performance&r {}",
+            fastfetch_path
+            .map(|path| format!("&a(Installed at {path})", path = path.display()))
+            .unwrap_or_else(|| "&c(Not installed)".to_owned())
+        );
         #[cfg(feature = "macchina")]
-        printc(
-            format!(
-                "- &bmacchina&r: Written in Rust, &nbest performance&r {installed_not_installed}\n",
-                installed_not_installed = macchina_path
-                    .map(|path| format!("&a(Installed at {path})", path = path.display()))
-                    .unwrap_or_else(|| "&c(Not installed)".to_owned())
-            ),
-            color_mode,
-        )
-        .context("failed to print message")?;
+        printc!("- &bmacchina&r: Written in Rust, &nbest performance&r {}\n",
+            macchina_path
+            .map(|path| format!("&a(Installed at {path})", path = path.display()))
+            .unwrap_or_else(|| "&c(Not installed)".to_owned())
+        );
 
         let choice = literal_input(
             "Your choice?",
@@ -1078,10 +1005,56 @@ fn create_config(
         backend.as_ref(),
     );
 
+    //////////////////////////////
+    // 8. Custom ASCII file
+    let mut custom_ascii_path: Option<String> = None;
+    clear_screen(Some(&title), color_mode, debug_mode).context("failed to clear screen")?;
+    let choice = literal_input(
+        "Do you want to specify a custom ASCII file?",
+        &["y", "n"],
+        "n",
+        true,
+        color_mode,
+    )
+    .context("failed to ask for choice input")?;
+
+    if choice == "y" {
+        loop {
+            let pth = input(Some("Path to custom ASCII file (must be UTF-8 encoded, empty to skip): "))?.trim().to_owned();
+            if pth.is_empty() {
+                printc!("&cNo path entered. Skipping custom ASCII file.");
+                break;
+            }
+
+            let pth_buf = PathBuf::from(&pth);
+            if !pth_buf.is_file() {
+                printc!("&cError: File not found at {pth}");
+                continue;
+            }
+
+            match fs::read_to_string(&pth_buf) {
+                Ok(_) => {
+                    custom_ascii_path = Some(pth);
+                    update_title(
+                        &mut title,
+                        &mut option_counter,
+                        "Custom ASCII file",
+                        custom_ascii_path.as_ref().unwrap(),
+                    );
+                    break;
+                }
+                Err(e) => {
+                    printc!("&cError: File is not UTF-8 encoded or an unexpected error occurred: {e}");
+                    continue;
+                }
+            }
+        }
+    }
+
     // Create config
     clear_screen(Some(&title), color_mode, debug_mode).context("failed to clear screen")?;
     let config = Config {
-        preset,
+        preset: preset.as_ref().to_string(),
         mode: color_mode,
         light_dark: Some(theme),
         auto_detect_light_dark: Some(det_bg.is_some()),
@@ -1091,6 +1064,7 @@ fn create_config(
         args: None,
         distro: logo_chosen,
         pride_month_disable: false,
+        custom_ascii_path,
     };
     debug!(?config, "created config");
 
