@@ -5,6 +5,7 @@ import argparse
 import datetime
 import importlib.util
 import json
+import os
 import random
 import traceback
 from itertools import permutations, islice
@@ -17,7 +18,7 @@ from .constants import *
 from .font_logo import get_font_logo
 from .models import Config
 from .neofetch_util import *
-from .presets import PRESETS
+from .presets import PRESETS, ColorProfile
 
 
 def check_config(path) -> Config:
@@ -318,9 +319,49 @@ def create_config() -> Config:
     backend = select_backend()
     update_title('Selected backend', backend)
 
+    ##############################
+    # 7. Custom ASCII file
+    custom_ascii_path = None
+    clear_screen(title)
+    if literal_input('Do you want to specify a custom ASCII file?', ['y', 'n'], 'n') == 'y':
+        while True:
+            path_input = input('Path to custom ASCII file (must be .txt and UTF-8 encoded): ').strip()
+            if not path_input:
+                printc('&cNo path entered. Skipping custom ASCII file.')
+                break
+
+            custom_path = Path(path_input)
+            if not custom_path.is_file():
+                printc(f'&cError: File not found at {path_input}')
+                if literal_input('Try again?', ['y', 'n'], 'y') == 'n':
+                    break
+                continue
+
+            if not custom_path.suffix == '.txt':
+                printc(f'&cError: File must have a .txt extension. Found {custom_path.suffix}')
+                if literal_input('Try again?', ['y', 'n'], 'y') == 'n':
+                    break
+                continue
+
+            try:
+                custom_path.read_text('utf-8')
+                custom_ascii_path = str(custom_path)
+                update_title('Custom ASCII file', custom_ascii_path)
+                break
+            except UnicodeDecodeError:
+                printc(f'&cError: File is not UTF-8 encoded.')
+                if literal_input('Try again?', ['y', 'n'], 'y') == 'n':
+                    break
+                continue
+            except Exception as e:
+                printc(f'&cAn unexpected error occurred: {e}')
+                if literal_input('Try again?', ['y', 'n'], 'y') == 'n':
+                    break
+                continue
+
     # Create config
     clear_screen(title)
-    c = Config(preset, color_system, light_dark, lightness, color_alignment, backend)
+    c = Config(preset, color_system, light_dark, lightness, color_alignment, backend, custom_ascii_path=custom_ascii_path)
 
     # Save config
     print()
@@ -338,7 +379,7 @@ def create_parser() -> argparse.ArgumentParser:
 
     parser.add_argument('-c', '--config', action='store_true', help=color(f'Configure hyfetch'))
     parser.add_argument('-C', '--config-file', dest='config_file', default=CONFIG_PATH, help=f'Use another config file')
-    parser.add_argument('-p', '--preset', help=f'Use preset', choices=list(PRESETS.keys()) + ['random'])
+    parser.add_argument('-p', '--preset', help=f'Use preset or comma-separated hex color list (e.g., "#ff0000,#00ff00,#0000ff")')
     parser.add_argument('-m', '--mode', help=f'Color mode', choices=['8bit', 'rgb'])
     parser.add_argument('-b', '--backend', help=f'Choose a *fetch backend', choices=['qwqfetch', 'neofetch', 'fastfetch', 'fastfetch-old'])
     parser.add_argument('--args', help=f'Additional arguments pass-through to backend')
@@ -451,7 +492,21 @@ def run():
     GLOBAL_CFG.is_light = config.light_dark == 'light'
 
     # Get preset
-    preset = PRESETS.get(config.preset)
+    preset = None
+    if config.preset in PRESETS:
+        preset = PRESETS.get(config.preset)
+    elif '#' in config.preset:
+        colors = [color.strip() for color in config.preset.split(',')]
+
+        for color in colors:
+            if not (color.startswith('#') and len(color) in [4, 7] and all(c in '0123456789abcdefABCDEF' for c in color[1:])):
+                print(f'Error: invalid hex color "{color}"')
+        preset = ColorProfile(colors)
+    else:
+        print(f'Preset should be a comma-separated list of hex colors, or one of the following: {", ".join(sorted(PRESETS.keys()))}')
+
+    if preset is None:
+        exit(1)
 
     # Lighten (args > config)
     if args.scale:
@@ -463,8 +518,14 @@ def run():
 
     # Run
     try:
-        asc = get_distro_ascii() if not args.ascii_file else Path(args.ascii_file).read_text("utf-8")
+        if config.custom_ascii_path:
+            asc = Path(config.custom_ascii_path).read_text("utf-8")
+        elif args.ascii_file:
+            asc = Path(args.ascii_file).read_text("utf-8")
+        else:
+            asc = get_distro_ascii()
         asc = config.color_align.recolor_ascii(asc, preset)
+        asc = '\n'.join(asc.split('\n')[1:])
         neofetch_util.run(asc, config.backend, config.args or '')
     except Exception as e:
         print(f'Error: {e}')
